'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _objectWithoutProperties2 = require('babel-runtime/helpers/objectWithoutProperties');

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

exports.getColumnsWidth = getColumnsWidth;
exports.parseColumns = parseColumns;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getColumnsWidth(columns) {
  return columns.reduce(function (memo, column) {
    return memo + column.width;
  }, 0);
}

function getColumnsFixed(columns) {
  var fixed = false;

  columns.some(function (column) {
    if (column.fixed) {
      fixed = column.fixed;
      return true;
    }
    return false;
  });

  return fixed;
}

function getLeftFixedEndIndex(columns) {
  // get index of the last left fixed column
  for (var i = columns.length - 1; i >= 0; i -= 1) {
    var fixed = columns[i].fixed;

    if (fixed === true || fixed === 'left') {
      return i;
    }
  }

  return -1;
}

function getRightFixedStartIndex(columns) {
  // get index of the first right fixed column
  for (var i = 0, l = columns.length; i < l; i += 1) {
    var fixed = columns[i].fixed;

    if (fixed === 'right') {
      return i;
    }
  }

  return -1;
}

function colneColumns(columns) {
  var widthMap = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var depth = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
  var parent = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : null;

  return columns.map(function (_ref, index) {
    var children = _ref.children,
        fixed = _ref.fixed,
        column = (0, _objectWithoutProperties3.default)(_ref, ['children', 'fixed']);

    column.depth = depth + 1;

    if (children && children.length) {
      column.children = colneColumns(children, widthMap, column.depth, column);
      column.width = getColumnsWidth(column.children);
      // calc group key
      column.key = column.children.map(function (_ref2) {
        var key = _ref2.key;
        return key;
      }).join('-');
      fixed = getColumnsFixed(column.children); // eslint-disable-line
    } else {
      // fix column key
      if (!column.key) {
        column.key = column.dataIndex;
      }

      if (widthMap[column.key]) {
        column.width = widthMap[column.key];
      }
    }

    column.isLastLeaf = index === columns.length - 1;
    column.isFirstLeaf = index === 0;

    if (fixed) {
      column.fixed = fixed;
    }

    if (parent) {
      column.parent = parent;
    }

    return column;
  });
}

function flattenColumns(columns) {
  var cols = [];
  columns.forEach(function (column) {
    if (column.children && column.children.length) {
      cols.push.apply(cols, (0, _toConsumableArray3.default)(flattenColumns(column.children)));
    } else {
      cols.push(column);
    }
  });

  return cols;
}

function finalCalc(columns, left, fixed) {
  var offsetLeft = left;
  columns.forEach(function (column) {
    column.left = offsetLeft;

    var current = column;
    var parent = current.parent;
    while (parent && current.isFirstLeaf) {
      parent.left = current.left;
      current = parent;
      parent = current.parent;
    }

    if (fixed && fixed !== column.fixed) {
      column.fixed = fixed;
      parent = column.parent;
      while (parent) {
        parent.fixed = fixed;
        parent = parent.parent;
      }
    }

    offsetLeft += column.width;
  });
}

function sortColumns(columns) {
  var orderMap = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var keys = (0, _keys2.default)(orderMap);
  if (keys.length) {
    var orders = keys.map(function (key) {
      return orderMap[key];
    });
    var cache = {};
    var result = [];

    columns.forEach(function (column) {
      var index = keys.indexOf(column.key);
      if (index >= 0) {
        cache[orders[index]] = column;
        result.push(null);
      } else {
        result.push(column);
      }
    });

    (0, _keys2.default)(cache).forEach(function (order) {
      var column = cache[order];
      result[order] = column;
      delete orderMap[column.key];
    });

    keys = (0, _keys2.default)(orderMap);
    if (keys.length) {
      result.some(function (column) {
        keys = (0, _keys2.default)(orderMap);
        if (keys.length && column.children) {
          column.children = sortColumns(column.children, orderMap);
        }

        return keys.length === 0;
      });
    }

    return result;
  }

  return columns;
}

function parseColumns(columns, _ref3) {
  var _ref3$widthMap = _ref3.widthMap,
      widthMap = _ref3$widthMap === undefined ? {} : _ref3$widthMap,
      _ref3$orderMap = _ref3.orderMap,
      orderMap = _ref3$orderMap === undefined ? {} : _ref3$orderMap;

  columns = colneColumns(columns, widthMap); // eslint-disable-line

  var leftFixedIndex = getLeftFixedEndIndex(columns);
  var rightFixedIndex = getRightFixedStartIndex(columns);

  var leftFixedColumns = leftFixedIndex >= 0 ? columns.slice(0, leftFixedIndex + 1) : [];
  var rightFixedColumns = rightFixedIndex > 0 ? columns.slice(rightFixedIndex, columns.length) : [];
  var scrollableColumns = columns.slice(leftFixedIndex >= 0 ? leftFixedIndex + 1 : 0, rightFixedIndex > 0 ? rightFixedIndex : columns.length);

  var cachedOrderMap = (0, _extends3.default)({}, orderMap);
  leftFixedColumns = sortColumns(leftFixedColumns, cachedOrderMap);
  scrollableColumns = sortColumns(scrollableColumns, cachedOrderMap);
  rightFixedColumns = sortColumns(rightFixedColumns, cachedOrderMap);

  var leftFixedLeafColumns = flattenColumns(leftFixedColumns);
  var scrollableLeafColumns = flattenColumns(scrollableColumns);
  var rightFixedLeafColumns = flattenColumns(rightFixedColumns);

  finalCalc(leftFixedLeafColumns, 0, 'left');
  finalCalc(scrollableLeafColumns, getColumnsWidth(leftFixedColumns), null);
  finalCalc(rightFixedLeafColumns, 0, 'right');

  var depth = Math.max(Math.max.apply(Math, (0, _toConsumableArray3.default)(leftFixedLeafColumns.map(function (column) {
    return column.depth;
  }))), Math.max.apply(Math, (0, _toConsumableArray3.default)(scrollableLeafColumns.map(function (column) {
    return column.depth;
  }))), Math.max.apply(Math, (0, _toConsumableArray3.default)(rightFixedLeafColumns.map(function (column) {
    return column.depth;
  }))));

  return {
    columns: columns,
    leftFixedColumns: leftFixedColumns,
    scrollableColumns: scrollableColumns,
    rightFixedColumns: rightFixedColumns,
    leftFixedLeafColumns: leftFixedLeafColumns,
    scrollableLeafColumns: scrollableLeafColumns,
    rightFixedLeafColumns: rightFixedLeafColumns,
    leftFixedColumnsWidth: getColumnsWidth(leftFixedColumns),
    scrollableColumnsWidth: getColumnsWidth(scrollableColumns),
    rightFixedColumnsWidth: getColumnsWidth(rightFixedColumns),
    depth: depth
  };
}