'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _PrefixIntervalTree = require('./struct/PrefixIntervalTree');

var _PrefixIntervalTree2 = _interopRequireDefault(_PrefixIntervalTree);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var BUFFER_ROWS = 5;
var NO_ROWS_SCROLL_RESULT = {
  index: 0,
  offset: 0,
  position: 0,
  contentHeight: 0
};

var FlexiGridScrollHelper = function () {
  function FlexiGridScrollHelper(rowCount, defaultRowHeight, viewportHeight, rowHeightGetter) {
    var _this = this;

    var defaultSubRowHeight = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 0;
    var subRowHeightGetter = arguments[5];
    (0, _classCallCheck3.default)(this, FlexiGridScrollHelper);

    this.getRowPosition = function (rowIndex) {
      _this.updateRowHeight(rowIndex);
      return _this.rowOffsets.sumUntil(rowIndex);
    };

    var defaultFullRowHeight = defaultRowHeight + defaultSubRowHeight;
    this.rowOffsets = _PrefixIntervalTree2.default.uniform(rowCount, defaultFullRowHeight);
    this.cachedHeights = [];
    for (var i = 0; i < rowCount; i += 1) {
      this.cachedHeights[i] = defaultFullRowHeight;
    }
    this.position = 0;
    this.rowCount = rowCount;
    this.viewportHeight = viewportHeight;
    this.contentHeight = rowCount * defaultFullRowHeight;

    this.rowHeightGetter = rowHeightGetter;
    this.subRowHeightGetter = subRowHeightGetter;
    this.fullRowHeightGetter = function (rowIndex) {
      var rowHeight = _this.rowHeightGetter ? _this.rowHeightGetter(rowIndex) : defaultRowHeight;
      var subRowHeight = _this.subRowHeightGetter ? _this.subRowHeightGetter(rowIndex) : defaultSubRowHeight;
      return rowHeight + subRowHeight;
    };

    this.updateHeightsInViewport(0, 0);
  }

  (0, _createClass3.default)(FlexiGridScrollHelper, [{
    key: 'setRowHeightGetter',
    value: function setRowHeightGetter(rowHeightGetter) {
      this.rowHeightGetter = rowHeightGetter;
    }
  }, {
    key: 'setSubRowHeightGetter',
    value: function setSubRowHeightGetter(subRowHeightGetter) {
      this.subRowHeightGetter = subRowHeightGetter;
    }
  }, {
    key: 'setViewportHeight',
    value: function setViewportHeight(viewportHeight) {
      this.viewportHeight = viewportHeight;
    }
  }, {
    key: 'getContentHeight',
    value: function getContentHeight() {
      return this.contentHeight;
    }
  }, {
    key: 'updateHeightsInViewport',
    value: function updateHeightsInViewport(firstRowIndex, firstRowOffset) {
      var top = firstRowOffset;
      var index = firstRowIndex;
      while (top <= this.viewportHeight && index < this.rowCount) {
        this.updateRowHeight(index);
        top += this.cachedHeights[index];
        index += 1;
      }
    }
  }, {
    key: 'updateHeightsAboveViewport',
    value: function updateHeightsAboveViewport(firstRowIndex) {
      var index = firstRowIndex - 1;
      while (index >= 0 && index >= firstRowIndex - BUFFER_ROWS) {
        var delta = this.updateRowHeight(index);
        this.position += delta;
        index -= 1;
      }
    }
  }, {
    key: 'updateRowHeight',
    value: function updateRowHeight(rowIndex) {
      if (rowIndex < 0 || rowIndex >= this.rowCount) {
        return 0;
      }

      var newHeight = this.fullRowHeightGetter(rowIndex);
      if (newHeight !== this.cachedHeights[rowIndex]) {
        var delta = newHeight - this.cachedHeights[rowIndex];
        this.rowOffsets.set(rowIndex, newHeight);
        this.cachedHeights[rowIndex] = newHeight;
        this.contentHeight += delta;
        return delta;
      }

      return 0;
    }
  }, {
    key: 'getRowAtEndPosition',
    value: function getRowAtEndPosition(rowIndex) {
      // We need to update enough rows above the selected one to be sure that when
      // we scroll to selected position all rows between first shown and selected
      // one have most recent heights computed and will not resize
      this.updateRowHeight(rowIndex);
      var currentRowIndex = rowIndex;
      var top = this.cachedHeights[currentRowIndex];
      while (top < this.viewportHeight && currentRowIndex >= 0) {
        currentRowIndex -= 1;
        if (currentRowIndex >= 0) {
          this.updateRowHeight(currentRowIndex);
          top += this.cachedHeights[currentRowIndex];
        }
      }

      var position = this.rowOffsets.sumTo(rowIndex) - this.viewportHeight;
      if (position < 0) {
        position = 0;
      }

      return position;
    }
  }, {
    key: 'scrollTo',
    value: function scrollTo(position) {
      if (this.rowCount === 0) {
        return NO_ROWS_SCROLL_RESULT;
      }

      if (position <= 0) {
        // If position less than or equal to 0 first row
        // should be fully visible on top.
        this.position = 0;
        this.updateHeightsInViewport(0, 0);

        return {
          index: 0,
          offset: 0,
          position: this.position,
          contentHeight: this.contentHeight
        };
      } else if (position >= this.contentHeight - this.viewportHeight) {
        // If position is equal to or greater than max scroll value, we need
        // to make sure to have bottom border of last row visible.
        var rowIndex = this.rowCount - 1;
        position = this.getRowAtEndPosition(rowIndex); // eslint-disable-line
      }

      this.position = position;

      var firstRowIndex = this.rowOffsets.greatestLowerBound(position);
      firstRowIndex = (0, _utils.clamp)(firstRowIndex, 0, Math.max(this.rowCount - 1, 0));

      var firstRowPosition = this.rowOffsets.sumUntil(firstRowIndex);
      var firstRowOffset = firstRowPosition - position;

      this.updateHeightsInViewport(firstRowIndex, firstRowOffset);
      this.updateHeightsAboveViewport(firstRowIndex);

      return {
        index: firstRowIndex,
        offset: firstRowOffset,
        position: this.position,
        contentHeight: this.contentHeight
      };
    }
  }, {
    key: 'scrollBy',
    value: function scrollBy(delta) {
      if (this.rowCount === 0) {
        return NO_ROWS_SCROLL_RESULT;
      }

      var firstRow = this.rowOffsets.greatestLowerBound(this.position);
      firstRow = (0, _utils.clamp)(firstRow, 0, Math.max(this.rowCount - 1, 0));
      var firstRowPosition = this.rowOffsets.sumUntil(firstRow);
      var rowIndex = firstRow;
      var position = this.position;

      var rowHeightChange = this.updateRowHeight(rowIndex);
      if (firstRowPosition !== 0) {
        position += rowHeightChange;
      }
      var visibleRowHeight = this.cachedHeights[rowIndex] - (position - firstRowPosition);

      if (delta >= 0) {
        while (delta > 0 && rowIndex < this.rowCount) {
          if (delta < visibleRowHeight) {
            position += delta;
            delta = 0; // eslint-disable-line
          } else {
            delta -= visibleRowHeight; // eslint-disable-line
            position += visibleRowHeight;
            rowIndex += 1;
          }
          if (rowIndex < this.rowCount) {
            this.updateRowHeight(rowIndex);
            visibleRowHeight = this.cachedHeights[rowIndex];
          }
        }
      } else if (delta < 0) {
        delta = -delta; // eslint-disable-line
        var invisibleRowHeight = this.cachedHeights[rowIndex] - visibleRowHeight;

        while (delta > 0 && rowIndex >= 0) {
          if (delta < invisibleRowHeight) {
            position -= delta;
            delta = 0; // eslint-disable-line
          } else {
            position -= invisibleRowHeight;
            delta -= invisibleRowHeight; // eslint-disable-line
            rowIndex -= 1;
          }
          if (rowIndex >= 0) {
            var change = this.updateRowHeight(rowIndex);
            invisibleRowHeight = this.cachedHeights[rowIndex];
            position += change;
          }
        }
      }

      var maxPosition = this.contentHeight - this.viewportHeight;
      position = (0, _utils.clamp)(position, 0, maxPosition);
      this.position = position;
      var firstRowIndex = this.rowOffsets.greatestLowerBound(position);
      firstRowIndex = (0, _utils.clamp)(firstRowIndex, 0, Math.max(this.rowCount - 1, 0));
      firstRowPosition = this.rowOffsets.sumUntil(firstRowIndex);
      var firstRowOffset = firstRowPosition - position;

      this.updateHeightsInViewport(firstRowIndex, firstRowOffset);
      this.updateHeightsAboveViewport(firstRowIndex);

      return {
        index: firstRowIndex,
        offset: firstRowOffset,
        position: this.position,
        contentHeight: this.contentHeight
      };
    }

    /**
     * Allows to scroll to selected row with specified offset. It always
     * brings that row to top of viewport with that offset
     */

  }, {
    key: 'scrollToRow',
    value: function scrollToRow(rowIndex, offset) {
      rowIndex = (0, _utils.clamp)(rowIndex, 0, Math.max(this.rowCount - 1, 0)); // eslint-disable-line
      offset = (0, _utils.clamp)(offset, -this.cachedHeights[rowIndex], 0); // eslint-disable-line
      var firstRow = this.rowOffsets.sumUntil(rowIndex);
      return this.scrollTo(firstRow - offset);
    }

    /**
     * Allows to scroll to selected row by bringing it to viewport with minimal
     * scrolling. This that if row is fully visible, scroll will not be changed.
     * If top border of row is above top of viewport it will be scrolled to be
     * fully visible on the top of viewport. If the bottom border of row is
     * below end of viewport, it will be scrolled up to be fully visible on the
     * bottom of viewport.
     */

  }, {
    key: 'scrollRowIntoView',
    value: function scrollRowIntoView(rowIndex) {
      rowIndex = (0, _utils.clamp)(rowIndex, 0, Math.max(this.rowCount - 1, 0)); // eslint-disable-line
      this.updateRowHeight(rowIndex);
      var rowBegin = this.rowOffsets.sumUntil(rowIndex);
      var rowEnd = rowBegin + this.cachedHeights[rowIndex];
      if (rowBegin < this.position) {
        return this.scrollTo(rowBegin);
      } else if (this.position + this.viewportHeight < rowEnd) {
        var position = this.getRowAtEndPosition(rowIndex);
        return this.scrollTo(position);
      }
      return this.scrollTo(this.position);
    }
  }]);
  return FlexiGridScrollHelper;
}();

exports.default = FlexiGridScrollHelper;
module.exports = exports['default'];