'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _Heap = require('./Heap');

var _Heap2 = _interopRequireDefault(_Heap);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var IntegerBufferSet = function () {
  function IntegerBufferSet() {
    (0, _classCallCheck3.default)(this, IntegerBufferSet);

    this.smallerComparator = function (lhs, rhs) {
      return lhs.value < rhs.value;
    };

    this.greaterComparator = function (lhs, rhs) {
      return lhs.value > rhs.value;
    };

    this.size = 0;
    this.smallValues = new _Heap2.default([], this.smallerComparator);
    this.largeValues = new _Heap2.default([], this.greaterComparator);
    this.valuePositionMap = {};
  }

  (0, _createClass3.default)(IntegerBufferSet, [{
    key: 'getSize',
    value: function getSize() {
      return this.size;
    }
  }, {
    key: 'getPositionForValue',
    value: function getPositionForValue(value) {
      var position = this.valuePositionMap[value];
      return position === undefined ? null : position;
    }
  }, {
    key: 'getNewPositionForValue',
    value: function getNewPositionForValue(value) {
      (0, _invariant2.default)(this.valuePositionMap[value] === undefined, "Shouldn't try to find new position for value already stored in BufferSet");

      var newPosition = this.size;
      this.size += 1;
      this.pushToHeaps(newPosition, value);
      this.valuePositionMap[value] = newPosition;
      return newPosition;
    }
  }, {
    key: 'replaceFurthestValuePosition',
    value: function replaceFurthestValuePosition(lowValue, highValue, newValue) {
      (0, _invariant2.default)(this.valuePositionMap[newValue] === undefined, "Shouldn't try to replace values with value already stored value in BufferSet");

      this.cleanHeaps();

      if (this.smallValues.isEmpty() || this.largeValues.isEmpty()) {
        // Threre are currently no values stored.
        // We will have to create new position for this value.
        return null;
      }

      var minValue = this.smallValues.peek().value;
      var maxValue = this.largeValues.peek().value;
      if (minValue >= lowValue && maxValue <= highValue) {
        // All values currently stored are necessary, we can't reuse any of them.
        return null;
      }

      var valueToReplace = void 0;
      if (lowValue - minValue > maxValue - highValue) {
        // minValue is further from provided range. We will reuse it's position.
        valueToReplace = minValue;
        this.smallValues.pop();
      } else {
        valueToReplace = maxValue;
        this.largeValues.pop();
      }
      var position = this.valuePositionMap[valueToReplace];
      delete this.valuePositionMap[valueToReplace];
      this.valuePositionMap[newValue] = position;
      this.pushToHeaps(position, newValue);

      return position;
    }
  }, {
    key: 'pushToHeaps',
    value: function pushToHeaps(position, value) {
      var item = {
        position: position,
        value: value
      };

      this.smallValues.push(item);
      this.largeValues.push(item);
    }
  }, {
    key: 'cleanHeaps',
    value: function cleanHeaps() {
      // We not usually only remove object from one heap while moving value.
      // Here we make sure that there is no stale data on top of heaps.
      this.cleanHeap(this.smallValues);
      this.cleanHeap(this.largeValues);
      var smallHeapSize = this.smallValues.getSize();
      var largeHeapSize = this.largeValues.getSize();
      var minHeapSize = Math.min(smallHeapSize, largeHeapSize);
      var maxHeapSize = Math.max(smallHeapSize, largeHeapSize);
      if (maxHeapSize > 10 * minHeapSize) {
        // There are many old values in one of heaps. We nned to
        // get rid of them to not use too avoid memory leaks
        this.reBuildHeaps();
      }
    }
  }, {
    key: 'reBuildHeaps',
    value: function reBuildHeaps() {
      var sourceHeap = this.smallValues.getSize() < this.largeValues.getSize() ? this.smallValues : this.largeValues;

      var newSmallValues = new _Heap2.default([], this.smallerComparator);
      var newLargeValues = new _Heap2.default([], this.greaterComparator);

      while (!sourceHeap.isEmpty()) {
        var item = sourceHeap.pop();
        // push all stil valid elements to new heaps
        if (this.valuePositionMap[item.value] !== undefined) {
          newSmallValues.push(item);
          newLargeValues.push(item);
        }
      }
      this.smallValues = newSmallValues;
      this.largeValues = newLargeValues;
    }
  }, {
    key: 'cleanHeap',
    value: function cleanHeap(heap) {
      while (!heap.isEmpty() && this.valuePositionMap[heap.peek().value] === undefined) {
        heap.pop();
      }
    }
  }]);
  return IntegerBufferSet;
}();

exports.default = IntegerBufferSet;
module.exports = exports['default'];